/* $Id: clihelp.c,v 1.5 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            clihelp.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Helper functions for clients, not network related.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdio.h>
#include <signal.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          snClientSetupSigHandlers
 *
 *  FUNCTION      Install signal handlers.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr snClientSetupSigHandlers(SNSigHandler termHandler);
 *
 *  INPUT         termHandler
 *                        function handling several kinds of signal
 *                        intended to abort the program. if NULL, no
 *                        handler is installed for these signals.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Installs a handler that will be called when the process
 *                receives one of several normal program termination
 *                signals.
 */
SNErr
snClientSetupSigHandlers(SNSigHandler termHandler)
{
    struct sigaction sact;

    sigemptyset(&sact.sa_mask);
    /* Signals are troublesome. On POSIX and System V, the default is
     * to not restart system calls that are interrupted by signals,
     * but rather make them fail. Thus we must specify that we want
     * the calls to be restarted. On BSD however, the default is to
     * restart the calls, so we should do nothing special. */
#ifdef SA_RESTART
    /* probably POSIX or System V. */
    sact.sa_flags = SA_RESTART;
#else
    /* probably BSD. */
    sact.sa_flags = 0;
#endif

    /* signals for program termination. */
    if (termHandler) {
	sact.sa_handler = termHandler;
	if (sigaction(SIGINT, &sact, NULL) < 0
	    || sigaction(SIGTERM, &sact, NULL) < 0
	    || sigaction(SIGQUIT, &sact, NULL) < 0
	    || sigaction(SIGABRT, &sact, NULL) < 0
	    || sigaction(SIGHUP, &sact, NULL) < 0)
	    return _snError(SN_ERR_ERRNO, "sicaction");
    }
    return _snError(SN_OK, NULL);
}
