/* $Id: commonhelp.c,v 1.3 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            commonhelp.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Helper functions for both clients and servers,
 *                  not network related.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <string.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          snChopLf
 *
 *  FUNCTION      Remove trailing '\n' from a string.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                int snChopLf(char *s);
 *
 *  INPUT         s       the string to remove a trailing '\n' from.
 *
 *  RETURNS       0 - nothing to remove, 1 - removal done.
 */
int
snChopLf(char *s)
{
    int pos;

    pos = strlen(s) - 1;
    if (pos >= 0 && s[pos] == '\n') {
	s[pos] = '\0';
	return 1;
    }
    return 0;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          snChopCrLf
 *
 *  FUNCTION      Remove trailing "\r\n" from a string.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                int snChopCrLf(char *s);
 *
 *  INPUT         s       the string to remove a trailing "\r\n" from.
 *
 *  RETURNS       0 - nothing to remove, 1 - removal done.
 */
int
snChopCrLf(char *s)
{
    int pos;

    pos = strlen(s) - 2;
    if (pos >= 0 && s[pos] == '\r' && s[pos + 1] == '\n') {
	s[pos] = '\0';
	return 1;
    }
    return 0;
}
