/* $Id: internal.c,v 1.10 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            internal.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Used internally.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                        P U B L I C    D A T A                          *
 *                                                                        *
 **************************************************************************/

SNErrorHandling _snErrorHandling = SN_REPORT_ERROR_AND_EXIT;
void (*_snErrorPrintf)(const char *format, ...) = NULL;
SNErr _snLastError = SN_OK;



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snError
 *
 *  FUNCTION      Handle an error as set up by snSetErrorHandling().
 *
 *  INPUT         err     the code of the error to handle.
 *                extra   any extra text to display along with the
 *                        error message, or NULL if no extra text should
 *                        be displayed. the error message will be of the
 *                        form "extra: error message" if extra is
 *                        not NULL.
 *
 *  RETURNS       An error code, or aborts the program if that's the
 *                wanted behaviour.
 *
 *  DESCRIPTION   This function is called for every error that occurs
 *                within the library functions. It possibly displays
 *                an error message, and possibly exits the program,
 *                according to the rules set by calling snSetErrorHandler().
 */
int
_snError(SNErr err, const char *extra)
{
    _snLastError = err;
    if (err == SN_OK)
	return err;
    if (_snErrorHandling == SN_RETURN_ERROR)
	return err;
    if (_snErrorHandling == SN_EXIT)
	exit(1);
    if (_snErrorPrintf) {
	if (extra)
	    _snErrorPrintf("%s: %s\n", extra, snErrToStr(err));
	else
	    _snErrorPrintf("%s\n", snErrToStr(err));
    } else {
	if (extra)
	    fprintf(stderr, "%s: %s\n", extra, snErrToStr(err));
	else
	    fprintf(stderr, "%s\n", snErrToStr(err));
    }
    if (_snErrorHandling == SN_REPORT_ERROR_AND_EXIT)
	exit(1);
    return err;
}
