/* $Id: sap-announce-dummy.c,v 1.3 2002/01/19 19:15:06 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            sap-announce-dummy.c
 *
 *  DESCRIPTION     Creates a dummy session, and annouces it using SAP
 *                  (Session Announcement Protocol) using multicast.
 *                  Mainly to test that multicast sending works.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <pwd.h>

#include <shhnet.h>

#define SAP_ADDR  "224.2.127.254"
#define SAP_PORT  "9875"

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          sapCreatePacket
 *
 *  FUNCTION      Create a dummy SAP packet including SDP session description.
 *
 *  INPUT         pack    buffer in which to store the packet.
 *
 *  OUTPUT        len     number of bytes entered in the buffer.
 */
static void
sapCreatePacket(unsigned char *pack, int *len)
{
    int           version = 1;
    int           messageType = 0; /* SDP announcement */
    int           encrypted = 0;
    int           compressed = 0;
    int           authLen = 0;
    int           msgIdHash = 0;
    unsigned long origSource;
    SNAddr        thisHostAddr;
    char          thisHost[100], thisHostDottedQuad[20], userName[20];
    unsigned long ntpNow;
    struct passwd *pw;
    char          *s =
	"v=0\n"   /* version */
	"o=%s %lu %lu IN IP4 %s\n"
	"s=Test session without transmission\n"
	"i=This is not a real session, just an announcement\n"
	"e=shh@thathost.com\n"
	"c=IN IP4 224.2.1.1/1\n"
	"t=%lu %lu\n"
	"m=audio 9999 RTP/AVT 0\n"
	;
    static time_t now;

    pw = getpwuid(getuid());
    strcpy(userName, pw->pw_name);
    if (!now)
	time(&now);
    ntpNow = (unsigned long) now + 2208988800UL;
    gethostname(thisHost, sizeof(thisHost));
    inHostNameToAddr(thisHost, &thisHostAddr);
    inHostAddrToStr(thisHostAddr, thisHostDottedQuad);
    origSource = inNetworkToHostAddr(thisHostAddr);

    pack[0] = (version & 7) << 5;
    pack[0] |= (messageType & 7) << 2;
    pack[0] |= (encrypted & 1) << 1;
    pack[0] |= (compressed & 1);

    pack[1] = authLen;

    pack[2] = (msgIdHash >> 8) & 0xFF;
    pack[3] = msgIdHash & 0xFF;

    pack[4] = (origSource >> 24) & 0xFF;
    pack[5] = (origSource >> 16) & 0xFF;
    pack[6] = (origSource >> 8) & 0xFF;
    pack[7] = origSource & 0xFF;

    *len = 8 + 4 * authLen + sprintf(pack + 8 + 4 * authLen, s,
				     userName, (unsigned long) now,
				     (unsigned long) now,
				     thisHostDottedQuad,
				     ntpNow, ntpNow + 300);
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
main(int argc, char *argv[])
{
    SNAddr        sapAddr;
    SNPort        sapPort;
    unsigned char buff[2048];
    int           sock, len, n, q;

    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, NULL);
    snInit();

    /* set up for multicast, with TTL for local subnet only. */
    udpMulticastOpen(SAP_ADDR, SAP_PORT, 1, &sock);

    inHostNameToAddr(SAP_ADDR, &sapAddr);
    inServiceToPort(SAP_PORT, "udp", &sapPort);

    for (q = 0; q < 5; q++) {
	sapCreatePacket(buff, &len);
	udpWriteTo(sock, buff, len, sapAddr, sapPort, &n);
	printf("multicasted %d bytes, sent %d bytes\n", len, n);
	sleep(5);
    }

    snFinish();
    return 0;
}
