/* $Id: sap-client.c,v 1.2 2002/01/19 19:15:06 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            sap-client.c
 *
 *  DESCRIPTION     Simple multicast reception program that recevies and
 *                  displays session announcements sent from programs such
 *                  as sdr.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include <shhnet.h>

#define SAP_ADDR  "224.2.127.254"
#define SAP_PORT  "9875"

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

static void
sapDescribePacket(const unsigned char *pack, int len)
{
    int    version;
    int    messageType;
    int    encrypted;
    int    compressed;
    int    authLen;
    int    msgIdHash;
    SNAddr origSource;
    char   s[80];
    int    textLen;

    if (len < 16) {
	printf("packet too short\n");
	return;
    }
    version = (pack[0] >> 5) & 7;
    messageType = (pack[0] >> 2) & 7;
    encrypted = (pack[0] >> 1) & 1;
    compressed = pack[0] & 1;
    authLen = pack[1];
    msgIdHash = (pack[2] << 8) | pack[3];
    origSource = pack[4];
    origSource = (origSource << 8) | pack[5];
    origSource = (origSource << 8) | pack[6];
    origSource = (origSource << 8) | pack[7];

    printf("SAP version %d, ", version);
    switch (messageType) {
      case 0:
	printf("SDP announcement");
	break;
      case 1:
	printf("SDP deletion");
	break;
      default:
	printf("unknown type");
    }
    if (encrypted)
	printf(", encrypted");
    if (compressed)
	printf(", compressed");
    printf("\n");
    if (origSource) {
	inHostAddrToName(origSource, s, sizeof(s));
	printf("originating source: %s\n", s);
    } else
	printf("no originating source given\n");
    printf("%d bytes of authentication information, msgIdHash: %X\n",
	   authLen * 4, msgIdHash);
    if (encrypted || compressed)
	return;
    printf("--- text of message ------\n");
    textLen = len - 8 - 4 * authLen;
    printf("%.*s", textLen, pack + 8 + 4 * authLen);
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
main(int argc, char *argv[])
{
    SNAddr        sapAddr, addr;
    SNPort        sapPort, port;
    unsigned char buff[2048];
    int           sock, n;
    char          s[80];

    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, NULL);
    snInit();

    udpMulticastOpen(SAP_ADDR, SAP_PORT, 0, &sock);

    for (;;) {
	/* TODO: i have not initiated the addresses, but it works anyway.
	 * possibly due to the joining of the multicast group. */
	addr = sapAddr;
	port = sapPort;
	udpReadFrom(sock, buff, sizeof(buff), &addr, &port, &n);
	inHostAddrToStr(addr, s);
	printf("read %d bytes from %s/%d\n",
	       n, s, (int) inNetworkToHostPort(port));
	sapDescribePacket(buff, n);
	printf("----------------------------------------------------\n");
    }
    snFinish();
    return 0;
}
