/* $Id: servopen.c,v 0.15 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            servopen.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     A function for opening server side connections
 *                  for both tcp and udp.
 *                  Used internally.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snSrvOpenAddrCommon
 *
 *  FUNCTION      Open a tcp- or udp-connection.
 *
 *  INPUT         type    The connection type: SN_PROTO_TCP or SN_PROTO_UDP
 *                addr    Local address to bind to.
 *                service A service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    she socket referencing this side of the
 *                        connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   The function will create a socket, and bind it to the
 *                given service/port.
 */
SNErr
_snSrvOpenAddrCommon(SNProto type, SNAddr addr, const char *service, int *sock)
{
    SNErr  err;
    SNPort port;
    char   *servicename = NULL;

    switch (type) {
      case SN_PROTO_TCP:
	servicename = "tcp";
	break;
      case SN_PROTO_UDP:
	servicename = "udp";
	break;
    }
    
    /* get the port number corresponding to the given service. */
    if ((err = inServiceToPort(service, servicename, &port)) != SN_OK)
	return err;

    /* create a socket. */
    if ((err = sockCreate(type, sock)) != SN_OK)
	return err;

    /* bind the local end, so clients have a valid address to talk to. */
    if ((err = sockBind(*sock, addr, port)) != SN_OK)
	return err;

    /* for TCP, start listening to incoming calls. */
    if (type == SN_PROTO_TCP)
	tcpListen(*sock, 5);

    return _snError(SN_OK, NULL);
}

/*-------------------------------------------------------------------------
 *
 *  NAME          _snSrvOpenCommon
 *
 *  FUNCTION      Open a tcp- or udp-connection.
 *
 *  INPUT         type    The connection type: SN_PROTO_TCP or SN_PROTO_UDP
 *                service A service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    she socket referencing this side of the
 *                        connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   The function will create a socket, and bind it to the
 *                given service/port.
 */
SNErr
_snSrvOpenCommon(SNProto type, const char *service, int *sock)
{
    return _snSrvOpenAddrCommon(type, SN_ANY_ADDR, service, sock);
}
