/* $Id: udpcli.c,v 0.11 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            udpcli.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions specific to the client side of
 *                  a udp-connection.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <sys/types.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          udpClientOpen
 *
 *  FUNCTION      Open a client side udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpClientOpen(const char *host, const char *service,
 *                                    int *sock);
 *
 *  INPUT         host    a hostname, or a hostaddress in dotted quad
 *                        notation.
 *                service a service name, or a port number (ascii). if a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    the socket referencing the connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Note that the socket is connected, so there is no
 *                need to specify addresses when sending packages.
 */
SNErr
udpClientOpen(const char *host, const char *service, int *sock)
{
    return _snCliOpenCommon(SN_PROTO_UDP, host, service, 1, sock);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          udpClientSock
 *
 *  FUNCTION      Get a locally bound socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpClientSock(int *sock);
 *
 *  OUTPUT        sock    the socket referencing the connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   This may be used to get an unconnected but bound
 *                UDP-socket. The socket is bound to some `random' port
 *                assigned by the operating system.
 */
SNErr
udpClientSock(int *sock)
{
    SNErr err;

    /* create and possibly bind a socket. */
    if ((err = sockCreate(SN_PROTO_UDP, sock)) != SN_OK)
	return err;
    return sockBind(*sock, SN_ANY_ADDR, SN_ANY_PORT);
}
