/* $Id: udpserv.c,v 0.9 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            udpserv.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions specific to the server side of
 *                  a udp-connection.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          udpServerOpen
 *
 *  FUNCTION      Open a server side udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpServerOpen(const char *service, int *sock);
 *
 *  INPUT         addr    Local address to bind to.
 *                service A service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    the socket referencing this side of the
 *                        `connection'.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   See _snSrvOpenCommon() in servopen.c
 */
SNErr
udpServerOpenAddr(SNAddr addr, const char *service, int *sock)
{
    return _snSrvOpenAddrCommon(SN_PROTO_UDP, addr, service, sock);
}

/*-------------------------------------------------------------------------
 *
 *  NAME          udpServerOpen
 *
 *  FUNCTION      Open a server side udp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr udpServerOpen(const char *service, int *sock);
 *
 *  INPUT         service A service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    the socket referencing this side of the
 *                        `connection'.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   See _snSrvOpenCommon() in servopen.c
 */
SNErr
udpServerOpen(const char *service, int *sock)
{
    return _snSrvOpenCommon(SN_PROTO_UDP, service, sock);
}
